# Helper objects to mock up various capabilities in frank

import datetime
import copy
import json


# Fake alerts, and generating alerts

reference_timestamp = datetime.datetime(2011, 1, 1)

class Severity(object):
    numbers2names = {
        1 : "VERYLOW",
        2 : "LOW",
        3 : "MEDIUM",
        4 : "HIGH",
        5 : "VERYHIGH"
    }

    def __init__(self, severity):
        self._severity = int(severity)
        self._name = self.numbers2names[self._severity]
        # It's how Alert does it....
        self.severity = self._severity
    def __cmp__(self, other): return cmp(self._severity, other._severity)
    def __str__(self): return self.name

    
class FakeAlert(object):
    def __init__(self, **params): self._params = params
    def equal_params(self, other_params): return self._params == other_params
    def __eq__(self, other): other.equal_params(self._params)
    """
    Represents alerts from the alert library.
    """
    def get_generator(self):
        return self._params['generator']
    def get_generator_version(self):
        return self._params['generator_version']
    def get_timestamp(self):
        return self._params['timestamp']
    def get_severity(self):
        return Severity(self._params['severity'])
    def get_tags(self):
        return self._params['tags']
    def get_fields_json(self):
        return self._params['data']
    

def default_state_transformer(idx, state):
    # Make an alert based on state, then create a new state for the
    # next alert.
    alert = FakeAlert(**state)
    new_state = copy.deepcopy(state)
    new_state['severity'] = (state['severity'] % 5) + 1
    new_state['timestamp'] = state['timestamp'] + datetime.timedelta(hours=1)
    return new_state, alert

alert_defaults = {
    "generator" : "org.cert.netsa.pipeline",
    "generator_version" : "0.1.2",
    "timestamp" : reference_timestamp,
    "severity": 1,
    "tags" : "type:watchlist name:test-watch",
    "data" : {
        "flows" : {"sip": "192.168.1.1",
                   "dip": "192.168.1.2",
                   "sport" : 123,
                   "dport" : 49152,
                   "proto" : 6},
    }
 }

def generate_alerts(num_alerts=10,
                    state_transformer=default_state_transformer):
    alerts = []
    state = alert_defaults
    for x in xrange(num_alerts):
        state, alert = state_transformer(x, state)
        alerts.append(alert)
    return alerts

def generate_alert(state_transformer=default_state_transformer):
    # Generate a single alert
    #
    # If the state transformer is the default, return a default
    # alert. If it's customized, transform the default and return the
    # transformed alert.
    if state_transformer == default_state_transformer:
        return generate_alerts(1)[0]
    else:
        return generate_alerts(2)[1]

def generate_default_alert(**customizations):
    merged_custom = copy.deepcopy(alert_defaults)
    merged_custom.update(customizations)
    return FakeAlert(**merged_custom)


# Fake database connection


class Fakedb_query(object):
    def __init__(self, query):
        self._query = query
    def __call__(self, conn, **kargs):
        return conn.execute(self, **kargs)
    def test_get_query(self): return self._query

class Fakedb_connection(object):
    def __init__(self, executed_statements, on_statement_execute):
        self._executed_statements = executed_statements
        self._on_statement_execute = on_statement_execute
    def execute(self, query, **kargs):
        try:
            self._executed_statements.append((query.test_get_query(), kargs))
        finally:
            if self._on_statement_execute is not None:
                return self._on_statement_execute(query, **kargs)
        

class FakeDatabase(object):
    # A stub for a preconfigured Database object the writer will get
    # from the DI
    def __init__(self,
                 executed_statements,
                 on_connect=None,
                 on_query=None,
                 on_statement_execute=None):
        self._executed_statements = executed_statements
        self._on_connect = on_connect
        self._on_query = on_query
        self._on_statement_execute = on_statement_execute
    def connect(self):
        try:
            return Fakedb_connection(self._executed_statements,
                                     self._on_statement_execute)
        finally:
            if self._on_connect is not None:
                self._on_connect()
                
    def query(self, query):
        try:
            return Fakedb_query(query)
        finally:
            if self._on_query is not None:
                self._on_query(query)
    

class FakeObject(object):
    def __init__(self, **events):
        self._events = events
    def call_method(self, meth_name, *args, **kargs):
        self._events[meth_name](*args, **kargs)
    
class FakeWriterOperations(FakeObject):
    def __init__(self, **events):
        self._alert_inserted = None
        self._bin_inserted = None
        self._bin_updated = None
        FakeObject.__init__(self, **events)
    def insert_alert(self, alert, alert_type, alert_name):
        self.call_method('insert_alert', alert, alert_type, alert_name)
    def update_bin(self, bin_timestamp, alert_type, alert_name):
        self.call_method('update_bin', bin_timestamp, alert_type, alert_name)
    def insert_bin(self, bin_timestamp, alert_type, alert_name):
        self.call_method('insert_bin', bin_timestamp, alert_type, alert_name)
    def commit(self):
        self.call_method('commit')
                

class FakeWriter(FakeObject):
    def __call__(self, alert):
        self.call_method('__call__', alert)

__all__ = """
FakeAlert
FakeDatabase
FakeObject
FakeWriter
FakeWriterOperations
generate_alert
generate_alerts
generate_default_alert
""".split()
